/********************************************************************************
 * Copyright (c) 2023 CEA-List
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0.
 *
 * SPDX-License-Identifier: EPL-2.0
 *
 ********************************************************************************/

#ifndef AIDGE_CORE_DATA_DATAPROVIDER_H_
#define AIDGE_CORE_DATA_DATAPROVIDER_H_

#include <cstddef>  // std::size_t
#include <memory>   // std::shared_ptr
#include <string>
#include <vector>   // std::vector

#include "aidge/data/Database.hpp"
#include "aidge/data/Data.hpp"



namespace Aidge {

/**
 * @brief Data Provider. Takes in a database and compose batches by fetching data from the given database.
 * @todo Implement Drop last batch option. Currently returns the last batch with less elements in the batch.
 * @todo Implement readRandomBatch to compose batches from the database with a random sampling startegy. Necessary for training.
 */
class DataProvider {
private:
    // Dataset providing the data to the dataProvider
    const Database& mDatabase;

    const std::size_t mNumberModality;
    std::vector<std::vector<std::size_t>> mDataSizes;
    std::vector<std::string> mDataBackends;
    std::vector<DataType> mDataTypes;

    // Desired size of the produced batches
    const std::size_t mBatchSize;

public:
    /**
     * @brief Constructor of Data Provider.
     * @param database database from which to load the data.
     * @param batchSize number of data samples per batch.
     */
    DataProvider(const Database& database, const std::size_t batchSize);

public:
    /**
     * @brief Create a batch for each data modality in the database. The returned batch contain the data as sorted in the database.
     * @param startIndex the starting index in the database to start the batch from.
     * @return a vector of tensors. Each tensor is a batch corresponding to one modality.
     */
    std::vector<std::shared_ptr<Tensor>> readBatch(const std::size_t startIndex) const;
};

} // namespace Aidge

#endif /* AIDGE_CORE_DATA_DATAPROVIDER_H_ */
