/********************************************************************************
 * Copyright (c) 2023 CEA-List
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0.
 *
 * SPDX-License-Identifier: EPL-2.0
 *
 ********************************************************************************/

#include <pybind11/pybind11.h>
#include <pybind11/stl.h>
#include <string>
#include <vector>
#include <array>

#include "aidge/backend/OperatorImpl.hpp"
#include "aidge/operator/OperatorTensor.hpp"
#include "aidge/operator/ReduceMean.hpp"
#include "aidge/utils/Types.h"

namespace py = pybind11;
namespace Aidge {

template <DimIdx_t DIM> void declare_ReduceMeanOp(py::module &m) {
  py::class_<ReduceMean_Op<DIM>, std::shared_ptr<ReduceMean_Op<DIM>>, Attributes, OperatorTensor>(
    m, ("ReduceMeanOp" + std::to_string(DIM) + "D").c_str(), py::multiple_inheritance())
    .def("get_inputs_name", &ReduceMean_Op<DIM>::getInputsName)
    .def("get_outputs_name", &ReduceMean_Op<DIM>::getOutputsName)
    .def("attributes_name", &ReduceMean_Op<DIM>::staticGetAttrsName)
    ;

  m.def(("ReduceMean" + std::to_string(DIM) + "D").c_str(), [](const std::vector<int>& axes,
                                                                DimSize_t keepDims,
                                                                const std::string& name) {
        AIDGE_ASSERT(axes.size() == DIM, "axes size [%ld] does not match DIM [%d]", axes.size(), DIM);

        return ReduceMean<DIM>(to_array<DIM>(axes.begin()), keepDims, name);
    }, py::arg("axes"),
       py::arg("keep_dims") = 1,
       py::arg("name") = "");
}


void init_ReduceMean(py::module &m) {
  declare_ReduceMeanOp<1>(m);
  declare_ReduceMeanOp<2>(m);
  declare_ReduceMeanOp<3>(m);

  // FIXME:
  // m.def("ReduceMean1D", static_cast<NodeAPI(*)(const char*, int, int, int const
  // (&)[1])>(&ReduceMean));
}
} // namespace Aidge
